# -*- coding: utf-8 -*-
import os
import shutil
import tempfile

import pytest
from click.testing import CliRunner


@pytest.fixture
def runner():
    yield CliRunner()


@pytest.fixture(autouse=True)
def isolate_filesystem():
    cwd = os.getcwd()
    t = tempfile.mkdtemp()
    os.chdir(t)
    try:
        yield t
    finally:
        os.chdir(cwd)
        try:
            shutil.rmtree(t)
        except (OSError, IOError):
            pass



# -*- coding: utf-8 -*-
from pathlib import Path

from click.testing import CliRunner

from umarkdown.cli import main


def test_cli_read_from_text_file(runner: CliRunner):
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("# Hello World")
        result = runner.invoke(main, ["hello.md"])
        assert result.exit_code == 0
        assert "<h1>Hello World</h1>\n" == result.output


def test_cli_write_to_file(runner: CliRunner):
    with runner.isolated_filesystem():
        result = runner.invoke(main, ["-", "hello.html"], input="# Hello World")
        assert result.exit_code == 0
        assert "<h1>Hello World</h1>\n" == Path("hello.html").read_text()


def test_cli_read_from_text_file_and_write_to_file(runner: CliRunner):
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("# Hello World")
        result = runner.invoke(main, ["hello.md", "hello.html"])
        assert result.exit_code == 0
        assert "<h1>Hello World</h1>\n" == Path("hello.html").read_text()


def test_cli_with_source_pos(runner: CliRunner):
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("# Hello World")
        result = runner.invoke(main, ["hello.md", "--sourcepos"])
        assert result.exit_code == 0
        assert '<h1 data-sourcepos="1:1-1:13">Hello World</h1>\n' == result.output


def test_cli_with_hard_breaks(runner: CliRunner):

    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello,\nWorld!")
        result = runner.invoke(main, ["hello.md"])
        assert result.exit_code == 0
        assert "<p>Hello,\nWorld!</p>\n" == result.output

    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello,\nWorld!")
        result = runner.invoke(main, ["hello.md", "--hardbreaks"])
        assert result.exit_code == 0
        assert "<p>Hello,<br />\nWorld!</p>\n" == result.output


def test_cli_with_no_breaks(runner: CliRunner):

    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello,\nWorld!")
        result = runner.invoke(main, ["hello.md"])
        assert result.exit_code == 0
        assert "<p>Hello,\nWorld!</p>\n" == result.output
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello,\nWorld!")
        result = runner.invoke(main, ["hello.md", "--nobreaks"])
        assert result.exit_code == 0
        assert "<p>Hello, World!</p>\n" == result.output


def test_cli_with_unsafe(runner: CliRunner):
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("<p>Hello World!</p>")
        result = runner.invoke(main, ["hello.md"])
        assert result.exit_code == 0
        assert "<!-- raw HTML omitted -->\n" == result.output

    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("<p>Hello World!</p>")
        result = runner.invoke(main, ["hello.md", "--unsafe"])
        assert result.exit_code == 0
        assert "<p>Hello World!</p>\n" == result.output


def test_cli_with_smart(runner: CliRunner):
    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello---")
        result = runner.invoke(main, ["hello.md"])
        assert result.exit_code == 0
        assert "<p>Hello---</p>\n" == result.output

    with runner.isolated_filesystem():
        with open("hello.md", "w") as f:
            f.write("Hello---")
        result = runner.invoke(main, ["hello.md", "--smart"])
        assert result.exit_code == 0
        assert "<p>Hello—</p>\n" == result.output



# -*- coding: utf-8 -*-
import platform
import sys
import timeit
from importlib.metadata import version  # type:ignore

import matplotlib.pyplot as plt
import numpy as np

TEST_DATA = (
    """
# h1 Heading
## h2 Heading
### h3 Heading
#### h4 Heading
##### h5 Heading
###### h6 Heading


## Horizontal Rules

___

---

***


## Typographic replacements

Enable typographer option to see result.

(c) (C) (r) (R) (tm) (TM) (p) (P) +-

test.. test... test..... test?..... test!....

!!!!!! ???? ,,  -- ---

"Smartypants, double quotes" and 'single quotes'


## Emphasis

**This is bold text**

__This is bold text__

*This is italic text*

_This is italic text_

~~Strikethrough~~


## Blockquotes


> Blockquotes can also be nested...
>> ...by using additional greater-than signs right next to each other...
> > > ...or with spaces between arrows.


## Lists

Unordered

+ Create a list by starting a line with `+`, `-`, or `*`
+ Sub-lists are made by indenting 2 spaces:
  - Marker character change forces new list start:
    * Ac tristique libero volutpat at
    + Facilisis in pretium nisl aliquet
    - Nulla volutpat aliquam velit
+ Very easy!

Ordered

1. Lorem ipsum dolor sit amet
2. Consectetur adipiscing elit
3. Integer molestie lorem at massa


1. You can use sequential numbers...
1. ...or keep all the numbers as `1.`

Start numbering with offset:

57. foo
1. bar


## Code

Inline `code`

Indented code

    // Some comments
    line 1 of code
    line 2 of code
    line 3 of code


Block code "fences"

```
Sample text here...
```

Syntax highlighting

``` js
var foo = function (bar) {
  return bar++;
};

console.log(foo(5));
```


## Links

[link text](http://example.com)

[link with title](http://example.com/ "title text!")

Autoconverted link http://example.com/


## Images

![Minion](https://octodex.github.com/images/minion.png)
![Stormtroopocat](https://octodex.github.com/images/stormtroopocat.jpg "The Stormtroopocat")

Like links, Images also have a footnote style syntax

![Alt text][id]

With a reference later in the document defining the URL location:

[id]: https://octodex.github.com/images/dojocat.jpg  "The Dojocat"

"""
    * 500
)


def main():
    print("Ultra Markdown parsing time compared to other popular Markdown parsers.")
    print()
    print("~~~~~~~~~~~~~")
    print("Test Machine:")
    print("~~~~~~~~~~~~~")
    print()
    uname_system, _, uname_release, uname_version, _, uname_processor = platform.uname()
    print(uname_system, uname_release, uname_processor, uname_version)
    print()
    print("~~~~~~~~")
    print("Version:")
    print("~~~~~~~~")
    print()
    print(
        "- {} {}".format(
            platform.python_implementation(), sys.version.replace("\n", "")
        )
    )
    print(f'- python-markdown    : {version("markdown")}')
    print(f'- python-markdown2   : {version("markdown2")}')
    print(f'- umarkdown          : {version("umarkdown")}')

    umarkdown_code = f"""
from umarkdown import markdown
markdown({TEST_DATA!r})
    """

    markdown_code = f"""
from markdown import markdown
markdown({TEST_DATA!r})
    """
    markdown2_code = f"""
from markdown2 import markdown
markdown({TEST_DATA!r})
    """
    umarkdown_time = timeit.timeit(stmt=umarkdown_code, number=10)
    python_markdown_time = timeit.timeit(stmt=markdown_code, number=10)
    python_markdown_2_time = timeit.timeit(stmt=markdown2_code, number=10)
    x = np.array(["Ultra Markdown", "Python-Markdown", "Python-Markdown2"])
    y = np.array(
        [1 / umarkdown_time, 1 / python_markdown_time, 1 / python_markdown_2_time]
    )
    plt.figure(figsize=(9, 7))
    plt.bar(x, y)
    plt.title(
        "Test Machine: \n"
        f"{uname_system} {uname_release} { uname_processor} { uname_version}\n\n"
        "Version: \n"
        "- {} {}\n".format(
            platform.python_implementation(), sys.version.replace("\n", "")
        )
        + f'- python-markdown: {version("markdown")}\n'
        f'- python-markdown2: {version("markdown2")}\n'
        f'- umarkdown: {version("umarkdown")}\n\n'
        "Test File Size: 1MB\n\n",
        fontweight="bold",
    )
    plt.ylabel("MB/s")
    plt.xlabel("Markdown parsers")
    plt.savefig(
        "docs/images/benchmarks.svg", bbox_inches="tight", dpi=1000, pad_inches=1
    )
    print("Benchmarks report saved to docs/images/benchmarks.svg")


if __name__ == "__main__":
    main()



#!/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
from glob import glob
from pathlib import Path

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext as _build_ext


class build_ext(_build_ext):
    def run(self):
        oldcwd = os.getcwd()
        os.chdir(os.path.join(os.path.dirname(__file__), "third_party", "cmark"))
        if os.path.exists("build"):
            shutil.rmtree("build")
        os.makedirs("build")
        os.chdir("build")
        subprocess.check_call(["cmake", ".."])
        os.chdir(oldcwd)
        return super().run()


setup(
    name="umarkdown",
    author="Kumar Aditya",
    author_email="",
    url="https://umarkdown.netlify.app/",
    description="Python wrapper of Markdown using CMark.",
    keywords=["Markdown", "CMark"],
    long_description=Path("README.md").read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    license="BSD License",
    packages=["umarkdown"],
    include_package_data=True,
    zip_safe=False,
    entry_points={"console_scripts": ["umarkdown=umarkdown.cli:main"]},
    ext_modules=[
        Extension(
            "umarkdown._internal",
            sources=["./umarkdown/_internal.c", *glob("./third_party/cmark/src/*.c")],
            include_dirs=["./third_party/cmark/src/", "./third_party/cmark/build/src/"],
        )
    ],
    cmdclass={"build_ext": build_ext},
    classifiers=[
        "Programming Language :: C",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Software Development :: Documentation",
        "Topic :: Text Processing :: Filters",
        "Topic :: Text Processing :: Markup :: HTML",
        "Topic :: Text Processing :: Markup :: Markdown",
        "License :: OSI Approved :: BSD License",
    ],
    use_scm_version=True,
    python_requires=">=3.7",
    extras_require={
        "cli": ["click==7.1.2"],
    },
)



# -*- coding: utf-8 -*-
from pathlib import Path

import pytest

from umarkdown import markdown


def test_umarkdown_with_text():
    with pytest.raises(TypeError):
        markdown()
    with pytest.raises(TypeError):
        markdown(text_file="not_exists.md")
    assert "<h1>Hello World</h1>\n" == markdown(text="# Hello World")


def test_umarkdown_with_text_file():
    with open("hello.md", "w") as f:
        f.write("# Hello World")
    assert "<h1>Hello World</h1>\n" == markdown(text_file="hello.md")


def test_umarkdown_with_output_file():
    markdown("# Hello World", output_file="hello.html")
    assert "<h1>Hello World</h1>\n" == Path("hello.html").read_text()


def test_umarkdown_with_text_file_and_output_file():
    with open("hello.md", "w") as f:
        f.write("# Hello World")
    assert markdown(text_file="hello.md", output_file="hello.html")
    assert "<h1>Hello World</h1>\n" == Path("hello.html").read_text()


def test_umarkdown_with_source_pos():
    assert '<h1 data-sourcepos="1:1-1:13">Hello World</h1>\n' == markdown(
        "# Hello World", source_pos=True
    )


def test_umarkdown_with_hard_breaks():
    assert "<p>Hello,\nWorld!</p>\n" == markdown("Hello,\nWorld!")
    assert "<p>Hello,<br />\nWorld!</p>\n" == markdown(
        "Hello,\nWorld!", hard_breaks=True
    )


def test_umarkdown_with_no_breaks():
    assert "<p>Hello,\nWorld!</p>\n" == markdown("Hello,\nWorld!")
    assert "<p>Hello, World!</p>\n" == markdown("Hello,\nWorld!", no_breaks=True)


def test_umarkdown_with_unsafe():
    assert "<!-- raw HTML omitted -->\n" == markdown("<p>Hello World!</p>")
    assert "<p>Hello World!</p>\n" == markdown("<p>Hello World!</p>", unsafe=True)


def test_umarkdown_with_smart():
    assert "<p>Hello---</p>\n" == markdown("Hello---")
    assert "<p>Hello—</p>\n" == markdown("Hello---", smart=True)



# -*- coding: utf-8 -*-
from typing import TextIO

try:
    import click
except ImportError:  # pragma:no cover
    print(
        """
            ===========================================
            || Ultra Markdown Command Line Interface ||
            ===========================================

             Ultra Markdown Cli dependencies not found.

        Make sure you have installed Ultra Markdown with cli.
            $ python -m pip install umarkdown[cli]
"""
    )
    exit(1)

from umarkdown import markdown


@click.command()
@click.argument(
    "file", required=True, type=click.File(mode="r", encoding="utf-8"), default="-"
)
@click.argument(
    "dest", required=True, type=click.File(mode="w", encoding="utf-8"), default="-"
)
@click.option(
    "--sourcepos",
    help="Include source position attribute.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
@click.option(
    "--hardbreaks",
    help="Treat newlines as hard line breaks.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
@click.option(
    "--nobreaks",
    help="Render soft line breaks as spaces.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
@click.option(
    "--unsafe",
    help="Render raw HTML and dangerous URLs.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
@click.option(
    "--smart",
    help="Use smart punctuation.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
@click.option(
    "--validate-utf8",
    help="Replace invalid UTF-8 sequences with U+FFFD.",
    is_flag=True,
    default=False,
    type=click.BOOL,
)
def main(
    file: TextIO,
    dest: TextIO,
    sourcepos: bool,
    hardbreaks: bool,
    nobreaks: bool,
    unsafe: bool,
    smart: bool,
    validate_utf8: bool,
):
    """
            \b
            ===========================================
            || Ultra Markdown Command Line Interface ||
            ===========================================
       Ultra Markdown is an ultrafast Markdown parser written in
          pure C with bindings for Python3.7+. It internally
    uses CMark, an ultrafast C library for parsing Markdown to HTML.

    """
    dest.write(
        markdown(
            text=file.read(),
            source_pos=sourcepos,
            hard_breaks=hardbreaks,
            no_breaks=nobreaks,
            unsafe=unsafe,
            smart=smart,
            validate_utf8=validate_utf8,
        )
    )



# -*- coding: utf-8 -*-
"""

Ultra Markdown
--------------

Ultra Markdown is an ultrafast Markdown parser written in
pure C with bindings for Python3.7+. It internally uses CMark,
an ultrafast C library for parsing Markdown to HTML.
Unlike others, Ultra Markdown is written using Python's C API
which makes it ultrafast for parsing Markdown.

Project Made and Maintained by Kumar Aditya.

License: BSD (see LICENSE.md for details).
"""

from ._internal import CMARK_VERSION, markdown

__all__ = ["markdown", "CMARK_VERSION"]



from typing import Optional, overload

CMARK_VERSION: str
@overload
def markdown(
    text: str,
    source_pos: Optional[bool] = ...,
    hard_breaks: Optional[bool] = ...,
    no_breaks: Optional[bool] = ...,
    smart: Optional[bool] = ...,
    unsafe: Optional[bool] = ...,
    validate_utf8: Optional[bool] = ...,
) -> str: ...
@overload
def markdown(
    text_file: str,
    source_pos: Optional[bool] = ...,
    hard_breaks: Optional[bool] = ...,
    no_breaks: Optional[bool] = ...,
    smart: Optional[bool] = ...,
    unsafe: Optional[bool] = ...,
    validate_utf8: Optional[bool] = ...,
) -> str: ...
@overload
def markdown(
    text: str,
    output_file: str,
    source_pos: Optional[bool] = ...,
    hard_breaks: Optional[bool] = ...,
    no_breaks: Optional[bool] = ...,
    smart: Optional[bool] = ...,
    unsafe: Optional[bool] = ...,
    validate_utf8: Optional[bool] = ...,
) -> bool: ...
@overload
def markdown(
    text_file: str,
    output_file: str,
    source_pos: Optional[bool] = ...,
    hard_breaks: Optional[bool] = ...,
    no_breaks: Optional[bool] = ...,
    smart: Optional[bool] = ...,
    unsafe: Optional[bool] = ...,
    validate_utf8: Optional[bool] = ...,
) -> bool: ...
